import { ArrowBack } from "@mui/icons-material";
import { Box, Link, Skeleton, Typography } from "@mui/material";
import { useEffect, useState } from "react";
import { Link as RouterLink, useLocation, useNavigate, useParams } from "react-router-dom";
import { useAccount, useChainId } from "wagmi";
import ReactGA from "react-ga4";

import PageTitle from "../../components/PageTitle/PageTitle";
import Metric from "../../components/Metric/Metric";
import TokenStack from "../../components/TokenStack/TokenStack";

import BondPrice from "./components/BondPrice";
import BondDiscount from "./components/BondDiscount";
import BondInputArea from "./components/BondInputArea";
import BondSettingsModal from "./components/BondSettingsModal";

import NotFound from "../NotFound/NotFound";

import { DecimalBigNumber } from "../../helpers/DecimalBigNumber";
import { NetworkId } from "../../constants";
import { formatNumber } from "../../helpers";

import { useLiveBonds } from "../../hooks/bonds";
import { useFtsoPrice } from "../../hooks/prices";

const BondModalContainer = ({ chainId, address, connect }) => {
    const { id } = useParams();
    const { liveBonds } = useLiveBonds(chainId);
    const bond = liveBonds.find(bond => bond.id === Number(id));

    if (!bond) return <NotFound />;

    return <BondModal chainId={chainId} bond={bond} address={address} connect={connect} />;
};

export const BondModal = ({ bond, chainId, address, connect }) => {
    const navigate = useNavigate();
    const { pathname } = useLocation();

    const [slippage, setSlippage] = useState(localStorage.getItem("bond-slippage") || "10");
    const [formatDecimals, setFormatDecimals] = useState(localStorage.getItem("bond-decimals") || "5");
    const [isSettingsOpen, setSettingsOpen] = useState(false);
    const [recipientAddress, setRecipientAddress] = useState(address);

    useEffect(() => {
        ReactGA.send({ hitType: "pageview", page: pathname });
    }, [])

    const setSlippageInner = (value) => {
        const maybeValue = parseFloat(value);
        if (!maybeValue || parseFloat(value) <= 100) {
            setSlippage(value);
            localStorage.setItem("bond-slippage", value);
        }
    }

    const setFormatDecimalsInner = (value) => {
        if (Number(value) <= 17) {
            setFormatDecimals(value);
            localStorage.setItem("bond-decimals", value);
        }
    }

    useEffect(() => {
        const handleKeyDown = (event) => {
            if (event.key === "Escape") isSettingsOpen ? setSettingsOpen(false) : navigate("/bonds");
        };
        window.addEventListener("keydown", handleKeyDown);
        return () => window.removeEventListener("keydown", handleKeyDown);
    }, [navigate, isSettingsOpen]);

    return (
        <Box>
            <PageTitle
                name={
                    <Box display="flex" flexDirection="row" alignItems="center">
                        <Link component={RouterLink} to="/bonds">
                            <Box display="flex" flexDirection="row">
                                <ArrowBack />
                                <Typography fontWeight="500" marginLeft="9.5px" marginRight="18px">
                                    Back
                                </Typography>
                            </Box>
                        </Link>

                        <TokenStack tokens={bond.quoteToken.icons} sx={{ fontSize: "27px" }} />
                        <Box display="flex" flexDirection="column" ml={1} justifyContent="center" alignItems="center">
                            <Typography variant="h4" fontWeight={500}>
                                {bond.quoteToken.name}
                            </Typography>
                        </Box>
                    </Box>
                }
            />

            <Box display="flex" flexDirection="column" alignItems="center">
                <BondSettingsModal
                    open={isSettingsOpen}
                    handleClose={() => setSettingsOpen(false)}
                    slippage={slippage}
                    recipientAddress={recipientAddress}
                    formatDecimals={formatDecimals}
                    onRecipientAddressChange={event => setRecipientAddress(event.currentTarget.value)}
                    onSlippageChange={event => setSlippageInner(event.currentTarget.value)}
                    onDecimalsChange={event => setFormatDecimalsInner(event.currentTarget.value)}
                />

                <Box display="flex" flexDirection="row" justifyContent="space-between" width={["100%", "70%"]} mt="24px">
                    <Metric
                        label={`Bond Price`}
                        metric={
                            bond.isSoldOut ? (
                                "--"
                            ) : (
                                <BondPrice
                                    price={bond.price.inBaseToken}
                                    symbol={bond.quoteToken.name}
                                />
                            )
                        }
                        />
                    <Metric
                        label="Market Price"
                        metric={
                            <TokenPrice
                                chainId={chainId}
                                token={bond.baseToken}
                                baseSymbol={bond.baseToken.name}
                                quoteSymbol={bond.quoteToken.name}
                            />
                        }
                        />
                    <Metric
                        label="Discount"
                        metric={<BondDiscount discount={bond.discount} textOnly />}
                        />
                </Box>

                <Box width="100%" mt="24px">
                    <BondInputArea
                        chainId={chainId}
                        bond={bond}
                        connect={connect}
                        address={address}
                        slippage={slippage}
                        recipientAddress={recipientAddress}
                        handleSettingsOpen={() => setSettingsOpen(true)}
                        formatDecimals={formatDecimals}
                    />
                </Box>
            </Box>
        </Box>
    );
};

const TokenPrice = ({
    token,
    chainId,
    quoteSymbol,
    baseSymbol,
}) => {
    const priceToken = useFtsoPrice(chainId);
    const sameToken = quoteSymbol === baseSymbol;

    const price = sameToken
        ? formatNumber(1, 2)
        : `${formatNumber(priceToken, 2)}`;

    return price ? (
        <>
            {price} {quoteSymbol}
        </>
    ) : (
        <Skeleton width={60} />
    );
};

export default BondModalContainer;
