import { Grid, Box, Typography, useTheme } from "@mui/material";
import { useAccount } from "wagmi";
import { useNavigate, createSearchParams } from "react-router-dom";

import Token from "../../../components/Token/Token";
import { SecondaryButton } from "../../../components/Button";
import { formatNumber, formatCurrency } from "../../../helpers";
import { DecimalBigNumber } from "../../../helpers/DecimalBigNumber";

import { useBalance } from "../../../hooks/tokens";
import {
    useFtsoPrice,
    useStnkPrice,
    useGhstPrice,
    useDaiPrice,
} from "../../../hooks/prices";

const TokenTab = ({ isMobileScreen, theme, tokenName, tokenUrl, tokenUrlParams, balance, price, description }) => {
    const navigate = useNavigate();
    const actualBalance = balance ? balance : new DecimalBigNumber(0, 0);

    return (
        <Box position="relative" width={`${isMobileScreen ? "100%" : "48%"}`}>
            <Box
                borderRadius="9px"
                padding="12px"
                sx={{ backgroundColor: theme.colors.paper.card }}
                display="flex"
                flexDirection="column"
                justifyContent="space-between"
            >
                <Box display="flex" gap={"3px"} alignItems="center" justifyContent="space-between">
                    <Box display="flex" gap="10px" alignItems="center">
                        <Token name={tokenName} />
                        <Typography fontSize="24px" fontWeight="500" lineHeight="33px">
                            {tokenName}
                        </Typography>
                    </Box>

                    <Box display="flex" flexDirection="column" alignItems="end" gap="3px">
                        <Typography fontSize="24px" fontWeight="500" lineHeight="33px">
                            {formatCurrency(price ? price : new DecimalBigNumber(0, 0), 2)}
                        </Typography>
                        <Typography fontSize="12px" fontWeight="450" lineHeight="12px" color={theme.colors.gray[40]}>
                            {formatNumber(actualBalance, 5)} {tokenName}
                        </Typography>
                    </Box>
                </Box>
                <>
                    <Box my="18px">
                        <Typography color={theme.colors.gray[40]} mt="9px">
                            {description}
                        </Typography>
                    </Box>

                    <Box display="flex" justifyContent="center" width="100%">
                        <SecondaryButton
                            onClick={() => navigate({
                                pathname: tokenUrl,
                                search: tokenUrlParams.toString()
                            })}
                            fullWidth
                        >
                            Get {tokenName}
                        </SecondaryButton>
                    </Box>
                </>
            </Box>
        </Box>
    )
}

const TokenInfo = ({ chainId, isMobileScreen }) => {
    const theme = useTheme();
    const { address } = useAccount();

    const ftsoPrice = useFtsoPrice(chainId);
    const stnkPrice = useStnkPrice(chainId);
    const ghstPrice = useGhstPrice(chainId);
    const daiPrice = useDaiPrice(chainId);

    const { balance: ftsoBalance, contractAddress: ftsoAddress } = useBalance(chainId, "FTSO", address);
    const { balance: stnkBalance, contractAddress: stnkAddress } = useBalance(chainId, "STNK", address);
    const { balance: ghstBalance, contractAddress: ghstAddress } = useBalance(chainId, "GHST", address);
    const { balance: daiBalance, contractAddress: daiAddress } = useBalance(chainId, "GDAI", address);

    return (
        <Grid container spacing={0} justifyContent={"center"}>
            <Box display="flex" flexWrap="wrap" justifyContent="space-between" mt="10px" gap="25px">
                <TokenTab
                    isMobileScreen={isMobileScreen}
                    tokenUrl="/dex/uniswap"
                    tokenUrlParams={createSearchParams({
                        from: `${daiAddress}`,
                        to: `${ftsoAddress}`,
                    })}
                    theme={theme}
                    tokenName="FTSO"
                    balance={ftsoBalance}
                    price={ftsoPrice}
                    description="FTSO is the native token of the ghostDAO protocol, fully backed by stablecoin reserves held in the ghostDAO treasury."
                />
                <TokenTab
                    isMobileScreen={isMobileScreen}
                    tokenUrl="/dex/uniswap"
                    tokenUrlParams={createSearchParams({
                        from: `${daiAddress}`,
                        to: `${stnkAddress}`,
                    })}
                    theme={theme}
                    tokenName="STNK"
                    balance={stnkBalance}
                    price={stnkPrice}
                    description="STNK is a receipt for staked FTSO, growing with staking rewards. When unstaked, it’s burned for FTSO at a 1:1 ratio."
                />
                <TokenTab
                    isMobileScreen={isMobileScreen}
                    tokenUrl="/dex/uniswap"
                    tokenUrlParams={createSearchParams({
                        from: `${daiAddress}`,
                        to: `${ghstAddress}`,
                    })}
                    theme={theme}
                    tokenName="GHST"
                    balance={ghstBalance}
                    price={ghstPrice}
                    description="GHST enables ghostDAO to have on-chain governance and to be truly cross-chain. GHST Price = FTSO Price x Current Index."
                />
                <TokenTab
                    isMobileScreen={isMobileScreen}
                    tokenUrl="/faucet"
                    tokenUrlParams=""
                    theme={theme}
                    tokenName="gDAI"
                    balance={daiBalance}
                    price={daiPrice}
                    description="FTSO is backed by a treasury reserve of crypto assets, with gDAI being the primary and most liquid asset."
                />
            </Box>
        </Grid>
    )
}

export default TokenInfo;
