import { useReadContract } from "wagmi";
import { simulateContract, writeContract, waitForTransactionReceipt } from "@wagmi/core";
import toast from "react-hot-toast";

import { config } from "../../config";

import { STAKING_ADDRESSES } from "../../constants/addresses";
import { abi as StakingAbi } from "../../abi/GhostStaking.json";

import { shorten } from "../../helpers";
import { DecimalBigNumber } from "../../helpers/DecimalBigNumber";

export const useCurrentIndex = (chainId) => {
    const { data: index, refetch } = useReadContract({
        abi: StakingAbi,
        address: STAKING_ADDRESSES[chainId],
        functionName: "index",
        scopeKey: `index-${chainId}`,
        chainId: chainId,
    });

    const currentIndexRaw = index ? index : 0n;
    const currentIndex = new DecimalBigNumber(currentIndexRaw, 9);

    return { currentIndex, refetch };
};

export const useEpoch = (chainId) => {
    const { data: epochRaw, refetch } = useReadContract({
        abi: StakingAbi,
        address: STAKING_ADDRESSES[chainId],
        functionName: "getEpoch",
        scopeKey: `getEpoch-${chainId}`,
        chainId: chainId,
    });

    const epoch = {
        length: Number(epochRaw && epochRaw.length ? epochRaw.length : 0),
        number: Number(epochRaw && epochRaw.number ? epochRaw.number : 0),
        end: Number(epochRaw && epochRaw.end ? epochRaw.end : 0),
        distribute: epochRaw && epochRaw.distribute ? new DecimalBigNumber(epochRaw.distribute, 9) : new DecimalBigNumber(0n, 9),
    };

    return { epoch, refetch };
}

export const useWarmupLength = (chainId) => {
    const { data: warmupLengthRaw } = useReadContract({
        abi: StakingAbi,
        address: STAKING_ADDRESSES[chainId],
        functionName: "warmupPeriod",
        scopeKey: `warmupPeriod-${chainId}`,
        chainId: chainId,
    });

    let warmupLength = new DecimalBigNumber(0n, 0);
    let warmupExists = false;

    if (warmupLengthRaw) {
        warmupLength = new DecimalBigNumber(warmupLengthRaw, 0);
        warmupExists = warmupLengthRaw > 0n;
    }

    return { warmupLength, warmupExists };
}

export const useWarmupInfo = (chainId, address) => {
    const { data: info, refetch } = useReadContract({
        abi: StakingAbi,
        address: STAKING_ADDRESSES[chainId],
        functionName: "getWarmupInfo",
        args: [address],
        scopeKey: `getWarmupInfo-${address}-${chainId}`,
        chainId: chainId,
    });

    const warmupInfo = {
        deposit: info ? new DecimalBigNumber(info.deposit, 9) : new DecimalBigNumber(0n, 9),
        shares: info ? info.shares : 0n,
        expiry: info ? Number(info.expiry) : 0,
    }

    return { warmupInfo, refetch }
}

export const stake = async (chainId, account, amount, isRebase, isClaim) => {
    const args = [
        amount,
        account,
        isRebase,
        isClaim
    ];
    const messages = {
        replacedMsg: "Staking transaction was replaced. Wait for inclusion please.",
        successMsg: `FTSO tokens staked successfully! Wait for the warmup period to claim your ${isRebase ? "STNK" : "GHST"}.`,
        errorMsg: "Staking transaction failed. Check logs for error detalization.",
    };
    await executeOnChainTransaction(
        chainId,
        "stake",
        args,
        account,
        messages
    );
}

export const unstake = async (chainId, account, amount, isTrigger, isRebase) => {
    const args = [
        amount,
        account,
        isTrigger,
        isRebase
    ];
    const messages = {
        replacedMsg: "Unstake transaction was replaced. Wait for inclusion please.",
        successMsg: `${isRebase ? "STNK" : "GHST"} tokens unstaked successfully! Check your FTSO balance on ${shorten(account)}.`,
        errorMsg: "Unstake transaction failed. Check logs for error detalization.",
    };
    await executeOnChainTransaction(
        chainId,
        "unstake",
        args,
        account,
        messages
    );
}

export const forfeit = async (chainId, account) => {
    const messages = {
        replacedMsg: "Forfeit transaction was replaced. Wait for inclusion please.",
        successMsg: `Tokens forfeited successfully! Check your FTSO balance on ${shorten(account)}.`,
        errorMsg: "Forfeit transaction failed. Check logs for error detalization.",
    };
    await executeOnChainTransaction(
        chainId,
        "forfeit",
        [],
        account,
        messages
    );
}

export const claim = async (chainId, account, isStnk) => {
    const args = [account, isStnk];
    const messages = {
        replacedMsg: "Claim transaction was replaced. Wait for inclusion please.",
        successMsg: `${isStnk ? "STNK" : "GHST"} tokens claimed successfully to ${shorten(account)}.`,
        errorMsg: "Claim transaction failed. Check logs for error detalization.",
    };
    await executeOnChainTransaction(
        chainId,
        "claim",
        args,
        account,
        messages
    );
}

export const unwrap = async (chainId, account, amount) => {
    const messages = {
        replacedMsg: "Unwrap transaction was replaced. Wait for inclusion please.",
        successMsg: `Tokens unwrapped successfully! Check your STNK balance on ${shorten(account)}.`,
        errorMsg: "Unwrap transaction failed. Check logs for error detalization.",
    };
    await executeOnChainTransaction(
        chainId,
        "unwrap",
        [account, amount],
        account,
        messages
    );
}

export const wrap = async (chainId, account, amount) => {
    const messages = {
        replacedMsg: "Wrap transaction was replaced. Wait for inclusion please.",
        successMsg: `Tokens wrapped successfully! Check your GHST balance on ${shorten(account)}.`,
        errorMsg: "Wrap transaction failed. Check logs for error detalization.",
    };
    await executeOnChainTransaction(
        chainId,
        "wrap",
        [account, amount],
        account,
        messages
    );
}

const executeOnChainTransaction = async (
    chainId,
    functionName,
    args,
    account,
    messages
) => {
    try {
        const { request } = await simulateContract(config, {
            abi: StakingAbi,
            address: STAKING_ADDRESSES[chainId],
            functionName,
            args,
            account,
            chainId
        });

        const txHash = await writeContract(config, request);
        await waitForTransactionReceipt(config, {
            hash: txHash,
            onReplaced: () => toast(messages.replacedMsg),
            chainId
        });

        toast.success(messages.successMsg);
    } catch (err) {
        console.error(err);
        toast.error(messages.errorMsg)
    }
}
